@@ -15,16 +15,6 @@ Booster <- R6::R6Class(
     params = list(),
     record_evals = list(),
     
-    # Finalize will free up the handles
-    finalize = function() {
-      .Call(
-        LGBM_BoosterFree_R
-        , private$handle
-      )
-      private$handle <- NULL
-      return(invisible(NULL))
-    },
-    
     # Initialize will create a starter booster
     initialize = function(params = list(),
                           train_set = NULL,
@@ -244,6 +234,12 @@ Booster <- R6::R6Class(
       
       self$params <- params
       
+      if (!is.null(self$params[["objective"]])) {
+        if (self$params[["objective"]] %in% c("mean_scale_regression", "gaussian_heteroscedastic")) {
+          private$is_mean_scale_regression <- TRUE
+        }
+      }
+      
       return(invisible(NULL))
       
     },
@@ -831,8 +827,8 @@ Booster <- R6::R6Class(
             fixed_effect <- NULL
           }
           
-        }# end Gaussian data
-        else{# non-Gaussian data
+        }# end Gaussian likelihood
+        else{# non-Gaussian likelihood
           
           y <- NULL
           # Either use raw_data or data loaded from file for determining fixed_effect_train
@@ -865,9 +861,19 @@ Booster <- R6::R6Class(
                                             , predcontrib = FALSE
                                             , header = header
                                             , reshape = FALSE )
+
+          if (private$gp_model$.__enclos_env__$private$num_sets_fe == 2) {
+            nt <- length(fixed_effect_train) / 2
+            fixed_effect_train <- fixed_effect_train[c((1:nt)*2 - 1, (1:nt)*2)]
+          }
           
           if (pred_latent) {
             
+            if (private$gp_model$.__enclos_env__$private$num_sets_fe == 2) {
+              np <- length(fixed_effect) / 2
+              fixed_effect <- fixed_effect[(1:np)*2 - 1] # take only predictions for mean
+            }
+            
             # Note: we don't need to provide the response variable y as this is saved
             #   in the gp_model ("in C++") for non-Gaussian data. y is only not NULL when
             #   the model was loaded from a file
@@ -897,7 +903,10 @@ Booster <- R6::R6Class(
             
           }# end pred_latent
           else {# predict response variable for non-Gaussian data
-            
+            if (private$gp_model$.__enclos_env__$private$num_sets_fe == 2) {
+              np <- length(fixed_effect) / 2
+              fixed_effect <- fixed_effect[c((1:np)*2 - 1, (1:np)*2)] # fixed effects predictions for mean and variance
+            }
             pred_resp <- private$gp_model$predict( group_data_pred = group_data_pred
                                                   , group_rand_coef_data_pred = group_rand_coef_data_pred
                                                   , gp_coords_pred = gp_coords_pred
@@ -927,6 +936,15 @@ Booster <- R6::R6Class(
                     response_var = response_var))
         
       }# end GPBoost prediction
+      else if (private$is_mean_scale_regression) {
+        preds <- predictor$predict(data = data, start_iteration = start_iteration
+          , num_iteration = num_iteration, rawscore = pred_latent, predleaf = predleaf
+          , predcontrib = predcontrib, header = header, reshape = reshape)
+        npred <- length(preds) / 2
+        pred_mean <- preds[(1:npred)*2 - 1]
+        pred_var <- exp(preds[(1:npred)*2])
+        return(list(pred_mean = pred_mean, pred_var = pred_var))
+      } 
       else {# no gp_model or predcontrib or ignore_gp_model
         return(
           predictor$predict(
@@ -985,6 +1003,18 @@ Booster <- R6::R6Class(
     higher_better_inner_eval = NULL,
     set_objective_to_none = FALSE,
     train_set_version = 0L,
+    is_mean_scale_regression = FALSE,
+    
+    # Finalize will free up the handles
+    finalize = function() {
+      .Call(
+        LGBM_BoosterFree_R
+        , private$handle
+      )
+      private$handle <- NULL
+      return(invisible(NULL))
+    },
+    
     # Predict data
     inner_predict = function(idx) {
       
