@@ -15,6 +15,7 @@
 #include <chrono>
 #include <ctime>
 #include <sstream>
+#include <cstring>
 
 namespace LightGBM {
 
@@ -61,6 +62,11 @@ namespace LightGBM {
 		num_iteration_for_pred_ = 0;
 		max_feature_idx_ = 0;
 		num_class_ = config->num_class;
+		if (objective_function != nullptr) {
+			if (std::strcmp(objective_function->GetName(), "mean_scale_regression") == 0) {
+				num_class_ = 2;
+			}
+		}
 		config_ = std::unique_ptr<Config>(new Config(*config));
 		early_stopping_round_ = config_->early_stopping_round;
 		es_first_metric_only_ = config_->first_metric_only;
@@ -89,7 +95,7 @@ namespace LightGBM {
 		num_tree_per_iteration_ = num_class_;
 		if (objective_function_ != nullptr) {
 			if (objective_function_->HasGPModel()) {
-				num_tree_per_iteration_ = objective_function_->GetNumSetsRE_GPModel();
+				num_tree_per_iteration_ = objective_function_->GetNumSetsFixedEffects();
 			}
 			else {
 				num_tree_per_iteration_ = objective_function_->NumModelPerIteration();
@@ -169,8 +175,8 @@ namespace LightGBM {
 		valid_metrics_.emplace_back();
 		for (const auto& metric : valid_metrics) {
 			valid_metrics_.back().push_back(metric);
-			if (metric->GetName()[0] == std::string("test_neg_log_likelihood") && 
-				!(objective_function_->UseGPModelForValidation())) {
+			if ((metric->GetName()[0] == std::string("test_neg_log_likelihood") || metric->GetName()[0] == std::string("crps_gaussian")) &&
+				!(objective_function_->UseGPModelForValidation()) && !(std::strcmp(objective_function_->GetName(), "mean_scale_regression") == 0)) {
 				calculate_residual_variance_ = true;
 			}
 		}
@@ -465,6 +471,9 @@ namespace LightGBM {
 				}
 				//Find optimal step length using a line search
 				if (objective_function_->HasGPModel() && config_->line_search_step_length) {
+					if (num_tree_per_iteration_ > 1) {
+						Log::Fatal("the option 'line_search_step_length' is currently no implemented for this likelihood ");
+					}
 					CHECK(!is_use_subset_);//if not true, this is not yet implemented, see 'UpdateScore' how to extend to this
 					CHECK(num_data_ == bag_data_cnt_);//if not true, this is not yet implemented, see 'UpdateScore' how to extend to this
 					std::vector<double> new_score(num_data_, 0.0);
@@ -592,18 +601,14 @@ namespace LightGBM {
 		// update training score
 		if (!is_use_subset_) {
 			train_score_updater_->AddScore(tree_learner_.get(), tree, cur_tree_id);
-
 			// we need to predict out-of-bag scores of data for boosting
 			if (num_data_ - bag_data_cnt_ > 0) {
 				train_score_updater_->AddScore(tree, bag_data_indices_.data() + bag_data_cnt_, num_data_ - bag_data_cnt_, cur_tree_id);
 			}
-
 		}
 		else {
 			train_score_updater_->AddScore(tree, cur_tree_id);
 		}
-
-
 		// update validation score
 		for (auto& score_updater : valid_score_updater_) {
 			score_updater->AddScore(tree, cur_tree_id);
